/*
 * PlayGameFrame.java
 * 
 * Copyright (c) 2004 ICEGT, Georgia Institute of Technology, Jeremy Goecks
 */
package edu.gatech.icegt.guesswho;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.Border;

/**
 * The interface for playing the GuessWho game.
 * @author Jeremy Goecks
 * @version 0.1
 */
public class PlayGameFrame extends JFrame 
	implements ActionListener {
	
	/** Number of rows to show. */
	protected static final int NUM_ROWS = 3;
	
	/** Number of columns to show. */
	protected static final int NUM_COLS = 5;	
	
	/** "All" string. */
	protected static final String ALL_STR = "all";
	
	/** The game engine. */
	protected GameEngine gameEngine;
	
	/** Fun fact label. */
	protected JLabel funFactLabel;
	
	/** The PersonPanel objects for the people in the game. */
	protected PersonPanel[] peoplePanels;
	
	/** Combo box used for gender selection. */
	protected JComboBox genderCb;
	
	/** Combo box used for hair-color selection. */
	protected JComboBox hairColorCb;
	
	/** List used for eye-color selection. */	
	protected JComboBox eyeColorCb;
	
	/** Combo box used for earrings selection. */
	protected JComboBox earringsCb;
	
	/** Answer's gender. */
	protected String answerGender;

	/** Answer's eye color. */
	protected String answerEyeColor;

	/** Answer's hair color. */
	protected String answerHairColor;

	/** Answer's answerHasEarrings. */
	protected Boolean answerHasEarrings;

	/** Label that shows the result of the last filtering action. */
	protected JLabel filteringResultLabel;
	
	/** Create frame. */
	public PlayGameFrame(GameEngine gameEngine) {
		//
		// Attribute init.
		//
		this.gameEngine = gameEngine;
		
		//
		// Init frame and start new game.
		//
		init();
	} // end PlayGameFrame()
	
	/**
	 * Initialize frame.
	 */
	private void init() {
		//
		// Reset game engine.
		//
		gameEngine.startNewGame(NUM_ROWS*NUM_COLS);
		
		//
		// Create panel to show people in game.
		//
		
		// Create panel, set layout, and add to frame.
		JPanel showPeoplePanel = new JPanel();
		GridLayout gridLayout = new GridLayout(NUM_ROWS, NUM_COLS);
		showPeoplePanel.setLayout(gridLayout);
		getContentPane().add(showPeoplePanel, BorderLayout.CENTER);
		
		// Add border to panel.
		Border etchedBorder = BorderFactory.createEtchedBorder();
		Border titledBorder = 
			BorderFactory.createTitledBorder(etchedBorder, "People");
		showPeoplePanel.setBorder(titledBorder);
		
		// Add people panels to showPeoplePanel.
		peoplePanels = new PersonPanel[NUM_ROWS*NUM_COLS];
		Person[] peopleInGame = gameEngine.getPeopleInCurGame();
		for (int i = 0; i < peoplePanels.length; i++) {
			peoplePanels[i] = new PersonPanel(this, peopleInGame[i], true);
			showPeoplePanel.add(peoplePanels[i]);
		} // end for
		
		//
		// Add fun fact panel, filter panel to WEST part of frame.
		//
		JPanel westPanel = new JPanel();
		westPanel.setLayout(new BoxLayout(westPanel, BoxLayout.Y_AXIS));
		JPanel 
			funFactPanel = createFunFactPanel(),
			filterPanel = createFilterPanel();
		westPanel.add(funFactPanel);
		westPanel.add(filterPanel);
		getContentPane().add(westPanel, BorderLayout.WEST);
		
		//
		// Set frame name.
		//
		setTitle("Guess Who: Play Game");
		
		// Pack and show.
		pack();
		show();
	} // end init()
	
	/**
	 * Start a new game.
	 * @return
	 */
	private void startNewGame() {
		//
		// Reset game engine and update people.
		//
		gameEngine.startNewGame(NUM_ROWS*NUM_COLS);
		
		//
		// Update fun fact.
		//
		funFactLabel.setText(gameEngine.getAnswersFunFact());
		
		// Update people to reflect current game.
		Person[] peopleInGame = gameEngine.getPeopleInCurGame();
		for (int i = 0; i < peopleInGame.length; i++) {
			peoplePanels[i].setPerson(peopleInGame[i]);
			peoplePanels[i].setIsMatch(true);
		} // end for
		
		//
		// Reset filters.
		//
		hairColorCb.setSelectedIndex(0);
		eyeColorCb.setSelectedIndex(0);
		genderCb.setSelectedIndex(0);
		earringsCb.setSelectedIndex(0);
		filteringResultLabel.setText("");
		
		//
		// Update answer attributes.
		//
		answerGender = answerHairColor = answerEyeColor = null;
		answerHasEarrings = null;
		
		// Update photos.
		updatePersonPhotosToMatchCurrentCriteria();
	} // end startNewGame()

	/**
	 * Method creates panel that shows fun fact for game's answer.
	 * @return
	 */
	protected JPanel createFunFactPanel() {
		//
		// Create panel.
		//
		JPanel funFactPanel = new JPanel();
		
		//
		// Add border to panel.
		//
		Border etchedBorder = BorderFactory.createEtchedBorder();
		Border titledBorder = 
			BorderFactory.createTitledBorder(etchedBorder, "Fun Fact: Whose is it?");
		funFactPanel.setBorder(titledBorder);
		
		//
		// Add fun fact as a label.
		//
		funFactLabel = new JLabel(gameEngine.getAnswersFunFact());
		funFactPanel.add(funFactLabel);
		
		return funFactPanel;
	} // end createFunFactPanel()
	
	/**
	 * Method creates panel that provides filtering options for user.
	 * @return
	 */
	protected JPanel createFilterPanel() {
		//
		// Create panel and layout.
		//
		JPanel filterPanel = new JPanel();
		GridBagLayout gbl = new GridBagLayout();
		filterPanel.setLayout(gbl);
		GridBagConstraints gbc = new GridBagConstraints();
		
		//
		// Add titled border to panel.
		//
		Border etchedBorder = BorderFactory.createEtchedBorder();
		Border titledBorder = 
			BorderFactory.createTitledBorder(etchedBorder, "Filtering");
		filterPanel.setBorder(titledBorder);
		
		//
		// Create and add gender labal and combo box.
		//
		
		// Add gender label.
		JLabel label = new JLabel("gender?");
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbl.setConstraints(label, gbc);
		filterPanel.add(label);
		
		// Create gender combo box.
		String[] cbEntries = { "all", "male", "female" };
		
		// Create combo box.
		genderCb = new JComboBox(cbEntries);
		genderCb.setSelectedIndex(0);
		
		// Add this object as list listener.
		genderCb.addActionListener(this);
		
		// Add combo box to panel.
		gbc.gridx = GridBagConstraints.RELATIVE;
		gbl.setConstraints(genderCb, gbc);
		filterPanel.add(genderCb);
		
		//
		// Add hair color label and combo box to panel.
		//
		
		// Add label.
		label = new JLabel("hair color:");
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbl.setConstraints(label, gbc);
		filterPanel.add(label);
		
		// Create list entries; "all" is the first entry and the other entries
		// are valid hair colors.
		cbEntries = new String[Person.VALID_HAIR_COLORS.length + 1];
		cbEntries[0] = ALL_STR;
		System.arraycopy(Person.VALID_HAIR_COLORS, 0, cbEntries, 1, 
							Person.VALID_HAIR_COLORS.length);
		
		// Create combo box.
		hairColorCb = new JComboBox(cbEntries);
		hairColorCb.setSelectedIndex(0);
		
		// Add this object as list listener.
		hairColorCb.addActionListener(this);
		
		// Add scroll pane to panel.
		gbc.gridx = GridBagConstraints.RELATIVE;
		gbl.setConstraints(hairColorCb, gbc);
		filterPanel.add(hairColorCb);
		
		//
		// Add eye color label and combo box to panel.
		//
		
		// Add label.
		label = new JLabel("eye color:");
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbl.setConstraints(label, gbc);
		filterPanel.add(label);
		
		// Create list entries; "all" is the first entry and the other entries
		// are valid eye colors.
		cbEntries = new String[Person.VALID_EYE_COLORS.length + 1];
		cbEntries[0] = ALL_STR;
		System.arraycopy(Person.VALID_EYE_COLORS, 0, cbEntries, 1, 
							Person.VALID_EYE_COLORS.length);
		
		// Create list.
		eyeColorCb = new JComboBox(cbEntries);
		eyeColorCb.setSelectedIndex(0);
		
		// Add this object as list listener.
		eyeColorCb.addActionListener(this);
		
		// Add combo box to panel.
		gbc.gridx = GridBagConstraints.RELATIVE;
		gbl.setConstraints(eyeColorCb, gbc);
		filterPanel.add(eyeColorCb);
		
		//
		// Create and add earrings labal and combo box.
		//
		
		// Add earrings label.
		label = new JLabel("earrings?");
		gbc.gridx = 0;
		gbc.gridy = 3;
		gbl.setConstraints(label, gbc);
		filterPanel.add(label);
		
		// Create gender combo box.
		cbEntries = new String[]{ "all", "yes", "no" };
		
		// Create combo box.
		earringsCb = new JComboBox(cbEntries);
		earringsCb.setSelectedIndex(0);
		
		// Add this object as list listener.
		earringsCb.addActionListener(this);
		
		// Add combo box to panel.
		gbc.gridx = GridBagConstraints.RELATIVE;
		gbl.setConstraints(earringsCb, gbc);
		filterPanel.add(earringsCb);
		
		//
		// Add filtering result label.
		//
		filteringResultLabel = new JLabel("");
		gbc.gridy = 4;
		gbc.gridwidth = 2;
		gbc.weighty = 100;
		gbl.setConstraints(filteringResultLabel, gbc);
		filterPanel.add(filteringResultLabel);
		
		return filterPanel;
	} // end createFilterPanel()
	
	/**
	 * Update person photos to match current filtering criteria.
	 */
	protected void updatePersonPhotosToMatchCurrentCriteria() {
		//
		// Get people that fit criteria and update person panels.
		//
		
		// Get people.
		Person[] peopleThatFitCriteria = 
			 gameEngine.getPeopleThatFitCriteria(answerGender, answerHairColor, 
			 										answerEyeColor, answerHasEarrings);
		
		// Update panels. We loop through the panels, determining if each person's
		// panel is in the set; we use the fact that both the panels and the
		// people that fit the criteria are ordered.
		int personIndex = 0;
		Person p = 
			(peopleThatFitCriteria.length != 0 ? peopleThatFitCriteria[0] : null);
		boolean fitsCriteria;
		for (int i = 0; i < peoplePanels.length; i++) {
			if (p == null) {
				// There are no more people in set; panel person's does not meet
				// criteria.
				peoplePanels[i].setIsMatch(false);
			} // end if
			else {
				// Determine if panel's person fits criteria.
				fitsCriteria = (peoplePanels[i].getPerson() == p);
				peoplePanels[i].setIsMatch(fitsCriteria);
				
				// Update p if we found panel with p.
				if (fitsCriteria)
					p = 
						(++personIndex != peopleThatFitCriteria.length ? 
							peopleThatFitCriteria[personIndex] : null);
			} // end else
		} // end for
	} // end updatePersonPhotosToMatchCurrentCriteria()
	
	/**
	 * Method called when user has guessed a person. Method shows dialog to
	 * indicate whether user is correct and, if he is correct, asks if he
	 * wanted to play again. 
	 * @param p
	 */
	protected void userGuessed(Person p) {
		// Is guess correct?
		boolean correct = gameEngine.isAnswer(p);
		
		//
		// Show dialog.
		//
		if (correct) {
			int returnVal = JOptionPane.showConfirmDialog(this, 
										  "Correct! " + p.getName() + " is the answer! Do you want to play again?",
										  "Correct!", JOptionPane.YES_NO_OPTION);
			
			if (returnVal == JOptionPane.YES_OPTION)
				startNewGame();
		} // end if
		else {
			JOptionPane.showMessageDialog(this, 
										  p.getName() + " is not the answer", 
										  "Incorrect guess", JOptionPane.INFORMATION_MESSAGE);
		} // end else
	} // end userGuessed()
	
	/**
	 * Method determines which combo box the user changed and updates the 
	 * interface accordingly.
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		//
		// Take action based on action's source and source's value.
		//
		JComboBox source = (JComboBox)e.getSource();
		String sourceValue = (String)source.getSelectedItem();
		if (source == genderCb) {
			if (sourceValue.equals("all"))
				answerGender = null;
			else {
				// Is guess correct?
				boolean correct = gameEngine.isAnswersGender(sourceValue);
				
				// If guess is correct, set criterion; otherwise criterion is
				// null.
				answerGender = (correct ? sourceValue : null);
					
				// Update filter result text.
				filteringResultLabel.setText("Answer's gender" + 
											 (correct ? " is " : " is not ") +
											 sourceValue);
			} // end else
		} // end if
		else if (source == hairColorCb) {
			if (sourceValue.equals("all"))
				answerGender = null;
			else {
				// Is guess correct?
				boolean correct = gameEngine.isAnswersHairColor(sourceValue);
				
				// If guess is correct, set criterion; otherwise criterion is
				// null.
				answerHairColor = (correct ? sourceValue : null);
					
				// Update filter result text.
				filteringResultLabel.setText("Answer's hair color" + 
											 (correct ? " is " : " is not ") +
											 sourceValue);
			} // end else
		} // end else if
		else if (source == eyeColorCb) {
			if (sourceValue.equals("all"))
				answerGender = null;
			else {
				// Is guess correct?
				boolean correct = gameEngine.isAnswersEyeColor(sourceValue);
				
				// If guess is correct, set criterion; otherwise criterion is
				// null.
				answerEyeColor = (correct ? sourceValue : null);
					
				// Update filter result text.
				filteringResultLabel.setText("Answer's eye color" + 
											 (correct ? " is " : " is not ") +
											 sourceValue);
			} // end else
		} // end else if
		else if (source == earringsCb) {
			if (sourceValue.equals("all"))
				answerGender = null;
			else {
				// Is guess correct?
				boolean guess = sourceValue.equals("yes");
				boolean correct = gameEngine.isAnswersHasEarrings(guess);
				
				// Set criterion regardlss; we do this because the answer is known
				// regardles of user's guess.
				answerHasEarrings = new Boolean(guess);
					
				// Update filter result text.
				String resultText = "";
				if (correct)
					resultText = 
						"Answer" + (guess ? " has " : " does not have ") +
						"earrings";
				else // !correct
					resultText = 
						"Answer" + (!guess ? " has " : " does not have ") +
						"earrings";
				
				filteringResultLabel.setText(resultText);
			} // end else
		} // end else if
		
		// Update photos.
		updatePersonPhotosToMatchCurrentCriteria();
	} // end actionPerformed()
} // enc class PlayGameFrame

